	name	MemKCT
	title	Dummy Kansas City Tracker TSR with Memory Output
;
;  This program does a minimal stupid emulation of the KCT TSR.
;  Instead of moving antennas, it accepts the antenna angles and
;  stores them in two places: in decimal ASCII for display in the
;  upper right corner of the screen, and in binary in a special
;  location in memory for access by other programs or hardware.
;
;  It assumes the screen is in 80-column mode based at 0xB8000.
;

; These definitions set where in memory the azimuth and elevation
; angles are stored.  I suggest the locations 0040:00f0 (physical
; bus address 004f0 hex) through 0040:00ff (physical bus address
; 004ff hex).  This area is designated as the "Intra-application
; Communication Area" by the IBM PC BIOS, and is intended for use
; by applications needing to exchange a few bytes of data.  This
; is too unsafe for use in production programs, so almost nobody
; uses it.
memory_segment          equ     0040h
memory_offset_azimuth   equ     00f0h
memory_offset_elevation equ     00f2h

_TEXT	segment word public 'CODE'

	ORG	100h			; this is a .COM file!

	assume	cs:_TEXT, ds:_TEXT, es:_TEXT, ss:_TEXT

install	proc	near

	mov	dx, offset isr	; get address of service routine
	mov	ax, 2563h	; set vector 0x63
	int	21h		; set up the interrupt vector

				; reserve memory and terminate.
	mov	dx,offset lastbyte	; get last address used
	add	dx,15		; round up
	mov	cl,4
	shr	dx,cl		; convert to paragraphs
	mov	ax,3100h	; TSR function code, good status
	int	21H		; go TSR!

install	endp

enabled	db	1		; enabled=1, disabled=0
azim	dw	0		; save target az and el
elev	dw	0


;******************************************************************
;  Here is the interrupt service routine.
;******************************************************************

	assume	ds:nothing, es:nothing, ss:nothing

isr:	jmp	dispatch

	ORG	isr + 10
	DB	"RotorDRV"	; signature string

dispatch proc	near
	sti
	cmp	ah,0		; dispatch according to function code
	je	readstat
	cmp	ah,3
	je	readpos
	cmp	ah,4
	je	settarg
	cmp	ah,9
	je	enable
	iret
dispatch endp

readstat proc	near
	mov	al,0		; status is always good.
	iret
readstat endp

readpos	proc	near
	mov	ax,cs:elev
	mov	cx,ax
	mov	bx,cs:azim
	mov	dx,bx
	iret
readpos	endp

enable	proc	near
	mov	enabled,al	; save the enable/disable flag.
	iret
enable	endp

;******************************************************************
; Function 04: Set Target position
; --------------------------------
; This function is used by the TSR to tell the tracker TSR where to
; point the antennas.  It is called periodically.
;
;    At Entry: AH=4
;              CX - target ELEV value in degrees (-90 to 269, signed binary)
;              DX - target AZIM value in degrees (0-359, binary)
;              Note: If either value is invalid the entire command will
;                    be ignored.
;
;    At Exit:  No parameters are returned
;*******************************************************************
; 123456789
; +000 +000
;  az   el

settarg	proc	near
	cmp	enabled,0	; don't do anything if disabled.
	je	st_exit

	pushf			; save direction flag

        mov     ax,memory_segment       ; address the memory location
        mov     es,ax
        mov     word ptr es:memory_offset_azimuth,dx    ; store azimuth
        mov     word ptr es:memory_offset_elevation,cx  ; store elevation

	mov	ax,0b800h
	mov	es,ax
	mov	di,140		; start in column 70
	push	dx		; save azimuth
	mov	cs:azim,dx
	push	cx		; save elevation
	mov	cs:elev,cx
	mov	cx,10
	cld	
	mov	ax,0720h	; spaces with normal attribute
	rep stosw		; fill with spaces

	mov	di,158
	std

	pop	cx		; retrieve elevation
	call	dispcx		; display elevation

	dec	di		; leave a space
	dec	di	

	pop	cx		; retrieve azimuth
	call	dispcx		; display azimuth

	popf			; restore direction flag

st_exit:	iret
settarg endp

; display a signed decimal number in CX at DI (decrementing)

dispcx	proc	near
	mov	si,0
	mov	ax,cx
	and	ax,ax		; is it negative?
	jge	dc_1		; no, go ahead
	neg	ax		; yes, make it positive
	inc	si		; remember it was negative
dc_1:	mov	dx,0
	mov	bx,10
	div	bx		; find next digit value
	mov	cx,ax		; save quotient
	mov	ax,dx
	add	ax,0730h	; convert to attribute:ascii pair
	stosw			; write to screen
	mov	ax,cx		; get back quotient
	and	ax,ax		; check for done
	jnz	dc_1		; not done, keep converting

	and	si,si		; was it negative?
	jz	dc_2		; nope, we're done
	mov	ax,072Dh	; yes, display minus sign
	stosw			; write to screen

dc_2:	ret

dispcx	endp

lastbyte	equ	this byte	; flag for end of memory used
_TEXT	ends

	end	install
